// PanelPlot2D class for Plot2D lab
// This class creates the panels for the plots

// (c) J. Dean Brock, Rebecca F. Bruce, 2001, 2004
// Department of Computer Science
// University of North Carolina at Asheville

// This is a package-specific class

package edu.unca.cs.csci201.LabAids ;

/**
 * Creates panels for student's plots within the Plot2D lab
 */
class PanelPlot2D extends edu.unca.cs.LabAids.PainterMux {
    
    private SpecPlot2D activeArtist ;

    /**
     * Contructor for PanelPlot2D
     * @param width Width of panel to <B>display</B> plots.
     * This is <B>not</B> the width of the frame.
     * @param height Height of panel to <B>display</B> plots.
     * This is <B>not</B> the height of the frame.
     */
    public PanelPlot2D(int width, int height) {
        super(width, height) ;
    }

    /**
     * Add a plotter
     * @param plotObject Object to the plotted.  Must follow the
     * SpecPlot2D interface.
     * @param plotName Name for button to select the plot
     */
    public void addPainter(SpecPlot2D plotObject, String plotName) {
        super.addPainter(new PainterPlot2D(plotObject), plotName) ;
    }
    
    private class PainterPlot2D implements edu.unca.cs.LabAids.Painter {
        SpecPlot2D painter2D ;
        
        private static final int Res = 100 ;
        private static final double invRes = 1.0 / Res ;
        
        private class SyncStateData {
            public double[] progOutputs ;
            public boolean runFailure = false ;
            public int maxCompleted = -1 ;
        } ;
        
        SyncStateData progState = new SyncStateData() ;
        
        private double pos2X(int i) {
            return -1.0 + (i/Res) + (i%Res)*invRes ;
        }
        
        public PainterPlot2D(SpecPlot2D NP2D) {
            painter2D = NP2D ;
            
            synchronized(progState) {
                progState.progOutputs = new double[2*Res+1] ;
            }
            
            Runnable computeNums = new Runnable() {
                public void run() {
                    boolean goodAnswers = true ; 
                    for (int i = 0; i <= 2*Res && goodAnswers; ++i) {
                        double x = pos2X(i) ;
                        try {
                            double y = painter2D.TestFunc(x) ;
                            if (!Double.isInfinite(y) && !Double.isNaN(y) && -1.0<=y && y<=10) {
                                synchronized(progState) {
                                    progState.progOutputs[i] = y ;
                                    progState.maxCompleted = i ;
                                 }
                            }
                            else {
                                goodAnswers = false ;
                                synchronized(progState) {
                                    progState.runFailure = true ;
                                }
                            }
                        } catch (Exception userExcp) {
                            goodAnswers = false ;
                            synchronized(progState) {
                                progState.runFailure = true ;
                            }
                        }
                    }
                }              
            } ;
            try {
                Thread computeThreads = new Thread(computeNums) ;
                // Let him run and hope he's done in time
                computeThreads.start() ;
            } catch (java.security.AccessControlException e) {
                // This hack allows the PainterMux to run under an applet
                // In that case, "infinite" loops with hang the applet
                computeNums.run() ;
            }
        }
        
        public void paintCanvas(java.awt.Graphics g, javax.swing.JComponent panel) {
            g = g.create() ;
            int centX = panel.getWidth()/2 ;
            int centY = panel.getHeight()/2 ;
            int minDim = ( (centX < centY) ? centX : centY ) * 9 / 10 ;
            
            g.setColor( java.awt.Color.orange ) ;
            int cornerPtX[] = { centX - minDim, centX + minDim, centX + minDim, centX - minDim } ;
            int cornerPtY[] = { centY + minDim, centY + minDim, centY - minDim, centY - minDim } ;
            g.drawPolygon(cornerPtX, cornerPtY, 4) ;
            g.drawLine(centX, centY + minDim, centX, centY - minDim) ;
            g.drawLine(centX - minDim, centY, centX + minDim, centY ) ;
            
            g.setColor( java.awt.Color.blue ) ;
            synchronized(progState) {
                double lastx = pos2X(0) ;
                double lasty = progState.progOutputs[0] ;
                for(int i = 1 ; i<=progState.maxCompleted ; ++i) {
                    double x = pos2X(i) ;
                    double y = progState.progOutputs[i] ;
                    if (-1.0<=lasty && lasty<=1.0 && -1.0<=y && y<=1.0)
                        g.drawLine(centX + (int)(lastx*minDim),
                                   centY - (int)(lasty*minDim),
                                   centX + (int)(x*minDim),
                                   centY - (int)(y*minDim)) ;
                    lastx = x ;
                    lasty = y ;
                }
                if (progState.maxCompleted<2*Res) {
                    double x = pos2X(progState.maxCompleted+1) ;
                    g.setColor(java.awt.Color.RED) ;
                    g.drawLine(centX + (int)(x*minDim),
                               centY + minDim,
                               centX + (int)(x*minDim),
                               centY - minDim) ;
                }
            }
        }
    }
}
